@@ -6,7 +6,7 @@
  * Descr.: FD to C macros - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -50,7 +50,7 @@
  * Descr.: parameter header file                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -105,7 +105,7 @@
  * Descr.: Wam architecture definition - description file                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -142,7 +142,7 @@
  * Descr.: Wam long type definition                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -185,7 +185,7 @@
  * Descr.: general configuration file (handled by autoconf) - header file  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -253,8 +253,21 @@
 /* Define if you have the sigsetjmp function */
 #define HAVE_SIGSETJMP 1
 
+/* NB: fpclassify is a macro, see BipsPl/arith_inl_c.c */
+/* Define if you have the fpclass function */
+/* #undef HAVE_FPCLASS */
+
+/* Define if you have the _fpclass function (windows) */
+/* #undef HAVE__FPCLASS */
+
+/* Define if you have the isnan function */
+#define HAVE_ISNAN 1
+
+/* Define if you have the isinf function */
+#define HAVE_ISINF 1
+
 /* Define if you have the asinh function */
-/* #undef HAVE_SINH */
+#define HAVE_ASINH 1
 
 /* Define if you have the acosh function */
 #define HAVE_ACOSH 1
@@ -307,14 +320,14 @@
 
 
 
-/* Define if the cpu is a mips */
-/* #undef M_mips */
+/* Define if the cpu is a mips32 */
+/* #undef M_mips32 */
 
 /* Define if the cpu is an alpha */
 /* #undef M_alpha */
 
-/* Define if the cpu is a sparc */
-/* #undef M_sparc */
+/* Define if the cpu is a sparc32 */
+/* #undef M_sparc32 */
 
 /* Define if the cpu is a sparc64 */
 /* #undef M_sparc64 */
@@ -322,12 +335,17 @@
 /* Define if the cpu is a ix86 */
 /* #undef M_ix86 */
 
-/* Define if the cpu is a powerpc */
-/* #undef M_powerpc */
+/* Define if the cpu is a ppc32 */
+/* #undef M_ppc32 */
 
 /* Define if the cpu is a x86-64 */
 #define M_x86_64 1
 
+/* Define if the cpu is a arm32 (armv7, armhf) */
+/* #undef M_arm32 */
+
+/* Define if the cpu is a arm4 (arch64) */
+/* #undef M_arm64 */
 
 /* Define if the OS is an SGI IRIX */
 /* #undef M_irix */
@@ -363,8 +381,8 @@
 #define M_bsd 1
 
 
-/* Define if the system is an mips/irix */
-/* #undef M_mips_irix */
+/* Define if the system is an mips32/irix */
+/* #undef M_mips32_irix */
 
 /* Define if the system is an alpha/linux */
 /* #undef M_alpha_linux */
@@ -396,26 +414,26 @@
 /* Define if the system is a ix86/solaris */
 /* #undef M_ix86_solaris */
 
-/* Define if the system is a sparc/solaris */
-/* #undef M_sparc_solaris */
+/* Define if the system is a sparc32/solaris */
+/* #undef M_sparc32_solaris */
 
-/* Define if the system is a sparc/sunos */
-/* #undef M_sparc_sunos */
+/* Define if the system is a sparc32/sunos */
+/* #undef M_sparc32_sunos */
 
-/* Define if the system is a sparc/bsd */
-/* #undef M_sparc_bsd */
+/* Define if the system is a sparc32/bsd */
+/* #undef M_sparc32_bsd */
 
 /* Define if the system is a sparc64/bsd */
 /* #undef M_sparc64_bsd */
 
-/* Define if the system is a powerpc/linux */
-/* #undef M_powerpc_linux */
+/* Define if the system is a ppc32/linux */
+/* #undef M_ppc32_linux */
 
-/* Define if the system is a powerpc/darwin */
-/* #undef M_powerpc_darwin */
+/* Define if the system is a ppc32/darwin */
+/* #undef M_ppc32_darwin */
 
-/* Define if the system is a powerpc/bsd */
-/* #undef M_powerpc_bsd */
+/* Define if the system is a ppc32/bsd */
+/* #undef M_ppc32_bsd */
 
 /* Define if the system is a x86-64/linux */
 /* #undef M_x86_64_linux */
@@ -426,12 +444,24 @@
 /* Define if the system is a x86-64/bsd */
 #define M_x86_64_bsd 1
 
+/* Define if the system is a x86-64/cygwin */
+/* #undef M_x86_64_cygwin */
+
 /* Define if the system is a x86-64/mingw */
 /* #undef M_x86_64_mingw */
 
 /* Define if the system is a x86-64/darwin */
 /* #undef M_x86_64_darwin */
 
+/* Define if the system is a arm32/linux */
+/* #undef M_arm32_linux */
+
+/* Define if the system is a arm64/linux */
+/* #undef M_arm64_linux */
+
+/* Define if the system is a arm64/darwin (e.g. Apple arm M1/MacOS)*/
+/* #undef M_arm64_darwin */
+
 
 /* Define if building universal (internal helper macro) */
 /* #undef AC_APPLE_UNIVERSAL_BUILD */
@@ -453,9 +483,9 @@
 
 #define PROLOG_DIALECT "gprolog"
 #define PROLOG_NAME "GNU Prolog"
-#define PROLOG_VERSION "1.4.5"
-#define PROLOG_DATE "Sep 26 2021"
-#define PROLOG_COPYRIGHT "Copyright (C) 1999-2021 Daniel Diaz"
+#define PROLOG_VERSION "1.5.0"
+#define PROLOG_DATE "Sep 28 2024"
+#define PROLOG_COPYRIGHT "Copyright (C) 1999-2024 Daniel Diaz"
 
 #define TOP_LEVEL "gprolog"
 #define GPLC "gplc"
@@ -464,12 +494,12 @@
 
 #define M_VENDOR "unknown"
 #define M_CPU "x86_64"
-#define M_OS "openbsd7.0"
+#define M_OS "openbsd7.6"
 
 #define CC "cc"
 #define CFLAGS_PREFIX_REG "-ffixed-%s"
-#define CFLAGS "-O2 -pipe  "
-#define CFLAGS_MACHINE " -fno-strict-aliasing"
+#define CFLAGS "-O2 -pipe   -Wno-char-subscripts"
+#define CFLAGS_MACHINE " -fno-strict-aliasing -fcommon"
 #define LDFLAGS ""
 #define LDLIBS "-lm"
 #define AS "as"
@@ -493,6 +523,7 @@
 #define LIB_BIPS_FD "libbips_fd.a"
 
 
+#define SIZEOF_INT 4
 #define SIZEOF_LONG 8
 #define SIZEOF_VOIDP 8
 
@@ -506,7 +537,7 @@
 #define HAVE_WORKING_SIGACTION 1
 
 /* Define if obj chain needs to reverse order of collected objects */
-/* #undef OBJ_CHAIN_REVERSE_ORDER */
+#define OBJ_CHAIN_REVERSE_ORDER 1
 
 /*-------------------------------------------------------------------------*
  * GNU Prolog                                                              *
@@ -516,7 +547,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -549,6 +580,16 @@
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
 
+/* for signals (see try_sigaction.c) */
+
+#if defined(M_ix86_sco)
+#define _XOPEN_SOURCE 700
+#define _XOPEN_SOURCE_EXTENDED
+#endif
+
+
+
+
 #define CPP_STR1(s) #s
 #define CPP_STR(s) CPP_STR1(s)
 
@@ -601,6 +642,16 @@
 #endif
 
 
+/* check printf arguments */
+
+#if defined(__GNUC__) && !defined(M_win64) /* gcc on mingw64 warns about PRIdPTR in x86_64_any.c ??? */
+#define ATTR_PRINTF(x) __attribute__((format(printf, x, x + 1)))
+#else
+#define ATTR_PRINTF(x)
+#endif
+
+
+
 
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
@@ -639,6 +690,7 @@
 #define strcasecmp                 stricmp
 #define strncasecmp                strnicmp
 #define spawnvp                    _spawnvp
+#define isnan                      _isnan
 #endif
 
 #ifndef F_OK
@@ -699,7 +751,7 @@
 
 
 
-#if defined(M_ix86_cygwin) || defined(M_ix86_sco)
+#if defined(__CYGWIN__) || defined(M_ix86_sco)
 #define Set_Line_Buf(s)            setvbuf(s, NULL, _IOLBF, 0)
 #elif defined(_WIN32)
 #define Set_Line_Buf(s)            setbuf(s, NULL)
@@ -712,7 +764,7 @@
 #define W32_GUI_CONSOLE
 #endif
 
-#ifdef M_sparc_sunos
+#ifdef M_sparc32_sunos
 #define __USE_FIXED_PROTOTYPES__
 #endif
 
@@ -727,7 +779,7 @@
 
 
 
-#if !defined(_WIN32) && !defined(__unix__)
+#if !defined(_WIN32) && !defined(__unix__) /* maybe too radical */
 #define __unix__
 #endif
 
@@ -774,8 +826,7 @@
 				/* Fast call macros */
 #if defined(M_ix86)
 
-/* FC_MAX_ARGS_IN_REGS can be decreased (0, 1, 2) - but the inline_asm_data
- * is compiled with 3, if changed, change inlined code in mapper */
+/* FC_MAX_ARGS_IN_REGS can be decreased (0, 1, 2) */
 
 #define COULD_COMPILE_FOR_FC
 #ifdef __GNUC__
@@ -1008,47 +1059,49 @@ typedef WamWord *WamWordP;
 
    /*--- Begin Register Generation ---*/
 
-register WamWord 		*pl_reg_bank asm ("r12");
+#ifdef ENGINE_FILE
 
-register WamWordP		TR  asm ("r13");
-register WamWordP		B   asm ("r14");
-register WamWordP		H   asm ("r15");
+       WamWord 			*pl_reg_bank;
 
+#else
 
-#define HB1			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+0])
-#define CP			(((WamCont  *) pl_reg_bank)[NB_OF_X_REGS+1])
-#define E			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+2])
-#define CS			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+3])
-#define S			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+4])
-#define STAMP			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+5])
-#define BCI			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+6])
-#define LSSA			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+7])
+extern WamWord 			*pl_reg_bank;
+
+#endif
+
+#define TR			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+0])
+#define B			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+1])
+#define H			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+2])
+#define HB1			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+3])
+#define CP			(((WamCont  *) pl_reg_bank)[NB_OF_X_REGS+4])
+#define E			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+5])
+#define CS			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+6])
+#define S			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+7])
+#define STAMP			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+8])
+#define BCI			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+9])
+#define LSSA			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+10])
 
 
 #define NB_OF_REGS          	11
-#define NB_OF_ALLOC_REGS    	3
-#define NB_OF_NOT_ALLOC_REGS	8
+#define NB_OF_ALLOC_REGS    	0
+#define NB_OF_NOT_ALLOC_REGS	11
 #define REG_BANK_SIZE       	(NB_OF_X_REGS+NB_OF_NOT_ALLOC_REGS)
 
 
 
 
-#define NB_OF_USED_MACHINE_REGS 4
+#define NB_OF_USED_MACHINE_REGS 0
 
 #ifdef ENGINE_FILE
 
 WamWord *save_reg_bank;
 
-WamWord pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS + 1];
-
 char *pl_reg_tbl[] = { "TR", "B", "H", "HB1", "CP", "E", "CS", "S", "STAMP", "BCI", "LSSA"};
 
 #else
 
 extern WamWord *save_reg_bank;
 
-extern WamWord pl_buff_signal_reg[];
-
 extern char *pl_reg_tbl[];
 
 #endif
@@ -1109,43 +1162,23 @@ extern char *pl_reg_tbl[];
 
 #define Save_Machine_Regs(buff_save) \
   do { \
-    buff_save[0] = (WamWord) pl_reg_bank; \
-    buff_save[1] = (WamWord) TR; \
-    buff_save[2] = (WamWord) B; \
-    buff_save[3] = (WamWord) H; \
   } while(0)
 
 
 #define Restore_Machine_Regs(buff_save) \
   do { \
-    pl_reg_bank = (WamWordP) buff_save[0]; \
-    TR = (WamWordP) buff_save[1]; \
-    B = (WamWordP) buff_save[2]; \
-    H = (WamWordP) buff_save[3]; \
   } while(0)
 
 
 
 
-#define Start_Protect_Regs_For_Signal \
-  do { \
-    Save_Machine_Regs(pl_buff_signal_reg); \
-    pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS] = 1; \
-  } while(0)
+#define Start_Protect_Regs_For_Signal
 
 
-#define Stop_Protect_Regs_For_Signal \
-  pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS] = 0; \
+#define Stop_Protect_Regs_For_Signal
 
 
-#define Restore_Protect_Regs_For_Signal \
-  do { \
-    if (pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS]) { \
-      Restore_Machine_Regs(pl_buff_signal_reg); \
-      Stop_Protect_Regs_For_Signal; \
-    } \
-    pl_reg_bank = save_reg_bank; \
-  } while(0)
+#define Restore_Protect_Regs_For_Signal
 
 
    /*--- End Register Generation ---*/
@@ -1366,7 +1399,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: general header file                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1404,7 +1437,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: FD Range Implementation - header file                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1442,7 +1475,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: boolean type definition - header file                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1539,7 +1572,7 @@ Range;
  * Descr.: FD Range Implementation - customizable header file              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1808,7 +1841,7 @@ char *Pl_Range_To_String(Range *range);
  * Descr.: FD instruction implementation - header file                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1956,6 +1989,7 @@ char *Pl_Range_To_String(Range *range);
 
 #define ENV_VAR_VECTOR_MAX         "VECTORMAX"
 #define DEFAULT_VECTOR_MAX         127
+#define VECTOR_MAX_LIMIT           1000000
 
 
 #define Fd_Variable_Is_Ground(fdv_adr) (Tag_Of(FD_Tag_Value(fdv_adr)) == INT)
